@@ -37,6 +37,8 @@ template<class T>
 inline void
 get_default_quantize_(long long& quant_min, long long& quant_max) noexcept
 {
+    OIIO_PRAGMA_WARNING_PUSH
+    OIIO_INTEL_PRAGMA(warning disable 173)
     if (std::numeric_limits<T>::is_integer) {
         quant_min = (long long)std::numeric_limits<T>::min();
         quant_max = (long long)std::numeric_limits<T>::max();
@@ -44,6 +46,7 @@ get_default_quantize_(long long& quant_min, long long& quant_max) noexcept
         quant_min = 0;
         quant_max = 0;
     }
+    OIIO_PRAGMA_WARNING_POP
 }
 
 
