@@ -4,13 +4,14 @@
 
 
 #include <cmath>
+#include <codecvt>
 #include <cstdarg>
 #include <cstdint>
 #include <cstdlib>
 #include <ctime>
 #include <iostream>
 #include <limits>
-#include <locale.h>
+#include <locale>
 #include <mutex>
 #include <numeric>
 #include <sstream>
@@ -267,7 +268,11 @@ bool
 Strutil::get_rest_arguments(const std::string& str, std::string& base,
                             std::map<std::string, std::string>& result)
 {
-    std::string::size_type mark_pos = str.find_first_of("?");
+    // Look for `?` as the start of REST arguments, but not if it's part
+    // of a `\\?\` special windows filename notation.
+    // See: https://docs.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation
+    size_t start  = starts_with(str, "\\\\?\\") ? 4 : 0;
+    auto mark_pos = str.find_first_of("?", start);
     if (mark_pos == std::string::npos) {
         base = str;
         return true;
@@ -275,9 +280,8 @@ Strutil::get_rest_arguments(const std::string& str, std::string& base,
 
     base             = str.substr(0, mark_pos);
     std::string rest = str.substr(mark_pos + 1);
-    std::vector<std::string> rest_tokens;
-    Strutil::split(rest, rest_tokens, "&");
-    for (const std::string& keyval : rest_tokens) {
+    auto rest_tokens = Strutil::splitsv(rest, "&");
+    for (auto keyval : rest_tokens) {
         mark_pos = keyval.find_first_of("=");
         if (mark_pos == std::string::npos)
             return false;
@@ -394,13 +398,37 @@ Strutil::wordwrap(string_view src, int columns, int prefix, string_view sep,
 
 
 
-// DEPRECATED(2.0) -- for link compatibility
 namespace Strutil {
-std::string
-wordwrap(string_view src, int columns, int prefix)
+
+// Define a locale-independent, case-insensitive comparison for all platforms.
+
+inline int
+strcasecmp(const char* a, const char* b)
 {
-    return wordwrap(src, columns, prefix, " ", "");
+#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) \
+    || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
+    return strcasecmp_l(a, b, c_loc);
+#elif defined(_WIN32)
+    return _stricmp_l(a, b, c_loc);
+#else
+#    error("need equivalent of strcasecmp_l on this platform");
+#endif
 }
+
+
+inline int
+strncasecmp(const char* a, const char* b, size_t size)
+{
+#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) \
+    || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
+    return strncasecmp_l(a, b, size, c_loc);
+#elif defined(_WIN32)
+    return _strnicmp_l(a, b, size, c_loc);
+#else
+#    error("need equivalent of strncasecmp_l on this platform");
+#endif
+}
+
 }  // namespace Strutil
 
 
@@ -408,43 +436,71 @@ wordwrap(string_view src, int columns, int prefix)
 bool
 Strutil::iequals(string_view a, string_view b)
 {
-    return boost::algorithm::iequals(a, b, std::locale::classic());
+    int asize = a.size();
+    int bsize = b.size();
+    if (asize != bsize)
+        return false;
+    return Strutil::strncasecmp(a.data(), b.data(), bsize) == 0;
 }
 
 
 bool
 Strutil::iless(string_view a, string_view b)
 {
-    return boost::algorithm::ilexicographical_compare(a, b,
-                                                      std::locale::classic());
+    size_t asize = a.size();
+    size_t bsize = b.size();
+    size_t size  = std::min(asize, bsize);
+    int c        = Strutil::strncasecmp(a.data(), b.data(), size);
+    // If c != 0, we could tell if a<b from the initial characters. But if c==0,
+    // the strings were equal up to the position of the lesser length, so
+    // whichever is smaller is "less" than the other.
+    return (c != 0) ? (c < 0) : (asize < bsize);
 }
 
 
 bool
 Strutil::starts_with(string_view a, string_view b)
 {
-    return boost::algorithm::starts_with(a, b);
+    size_t asize = a.size();
+    size_t bsize = b.size();
+    if (asize < bsize)
+        return false;
+    return strncmp(a.data(), b.data(), bsize) == 0;
 }
 
 
 bool
 Strutil::istarts_with(string_view a, string_view b)
 {
-    return boost::algorithm::istarts_with(a, b, std::locale::classic());
+    size_t asize = a.size();
+    size_t bsize = b.size();
+    if (asize < bsize)  // a can't start with b if a is smaller
+        return false;
+    return Strutil::strncasecmp(a.data(), b.data(), bsize) == 0;
 }
 
 
 bool
 Strutil::ends_with(string_view a, string_view b)
 {
-    return boost::algorithm::ends_with(a, b);
+    size_t asize = a.size();
+    size_t bsize = b.size();
+    if (asize < bsize)  // a can't start with b if a is smaller
+        return false;
+    return strncmp(a.data() + asize - bsize, b.data(), bsize) == 0;
+    // return boost::algorithm::ends_with(a, b);
 }
 
 
 bool
 Strutil::iends_with(string_view a, string_view b)
 {
-    return boost::algorithm::iends_with(a, b, std::locale::classic());
+    size_t asize = a.size();
+    size_t bsize = b.size();
+    if (asize < bsize)  // a can't start with b if a is smaller
+        return false;
+    return strncasecmp(a.data() + asize - bsize, b.data(), bsize) == 0;
+    // return boost::algorithm::iends_with(a, b, std::locale::classic());
 }
 
 
@@ -540,15 +596,17 @@ Strutil::to_upper(std::string& a)
 bool
 Strutil::StringIEqual::operator()(const char* a, const char* b) const noexcept
 {
-    return boost::algorithm::iequals(a, b, std::locale::classic());
+    return Strutil::strcasecmp(a, b) == 0;
+    // return boost::algorithm::iequals(a, b, std::locale::classic());
 }
 
 
 bool
 Strutil::StringILess::operator()(const char* a, const char* b) const noexcept
 {
-    return boost::algorithm::ilexicographical_compare(a, b,
-                                                      std::locale::classic());
+    return Strutil::strcasecmp(a, b) < 0;
+    // return boost::algorithm::ilexicographical_compare(a, b,
+    //                                                   std::locale::classic());
 }
 
 
@@ -597,16 +655,16 @@ split_whitespace(string_view str, std::vector<string_view>& result,
     // Implementation inspired by Pystring
     string_view::size_type i, j, len = str.size();
     for (i = j = 0; i < len;) {
-        while (i < len && ::isspace(str[i]))
+        while (i < len && Strutil::isspace(str[i]))
             i++;
         j = i;
-        while (i < len && !::isspace(str[i]))
+        while (i < len && !Strutil::isspace(str[i]))
             i++;
         if (j < i) {
             if (--maxsplit <= 0)
                 break;
             result.push_back(str.substr(j, i - j));
-            while (i < len && ::isspace(str[i]))
+            while (i < len && Strutil::isspace(str[i]))
                 i++;
             j = i;
         }
@@ -744,18 +802,46 @@ Strutil::replace(string_view str, string_view pattern, string_view replacement,
 
 
 
-#ifdef _WIN32
+// Conversion functions between UTF-8 and UTF-16 for windows.
+//
+// For historical reasons, the standard encoding for strings on windows is
+// UTF-16, whereas the unix world seems to have settled on UTF-8.  These two
+// encodings can be stored in std::string and std::wstring respectively, with
+// the caveat that they're both variable-width encodings, so not all the
+// standard string methods will make sense (for example std::string::size()
+// won't return the number of glyphs in a UTF-8 string, unless it happens to
+// be made up of only the 7-bit ASCII subset).
+//
+// The standard windows API functions usually have two versions, a UTF-16
+// version with a 'W' suffix (using wchar_t* strings), and an ANSI version
+// with a 'A' suffix (using char* strings) which uses the current windows
+// code page to define the encoding.  (To make matters more confusing there is
+// also a further "TCHAR" version which is #defined to the UTF-16 or ANSI
+// version, depending on whether UNICODE is defined during compilation.
+// This is meant to make it possible to support compiling libraries in
+// either unicode or ansi mode from the same codebase.)
+//
+// Using std::string as the string container (as in OIIO) implies that we
+// can't use UTF-16.  It also means we need a variable-width encoding to
+// represent characters in non-Latin alphabets in an unambiguous way; the
+// obvious candidate is UTF-8.  File paths in OIIO are considered to be
+// represented in UTF-8, and must be converted to UTF-16 before passing to
+// windows API file opening functions.
+//
+// On the other hand, the encoding used for the ANSI versions of the windows
+// API is the current windows code page.  This is more compatible with the
+// default setup of the standard windows command prompt, and may be more
+// appropriate for error messages.
+
 std::wstring
 Strutil::utf8_to_utf16(string_view str) noexcept
 {
-    std::wstring native;
-
-    native.resize(
-        MultiByteToWideChar(CP_UTF8, 0, str.data(), str.length(), NULL, 0));
-    MultiByteToWideChar(CP_UTF8, 0, str.data(), str.length(), &native[0],
-                        (int)native.size());
-
-    return native;
+    try {
+        std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> conv;
+        return conv.from_bytes(str.data(), str.data() + str.size());
+    } catch (const std::exception&) {
+        return std::wstring();
+    }
 }
 
 
@@ -763,16 +849,13 @@ Strutil::utf8_to_utf16(string_view str) noexcept
 std::string
 Strutil::utf16_to_utf8(const std::wstring& str) noexcept
 {
-    std::string utf8;
-
-    utf8.resize(WideCharToMultiByte(CP_UTF8, 0, str.data(), str.length(), NULL,
-                                    0, NULL, NULL));
-    WideCharToMultiByte(CP_UTF8, 0, str.data(), str.length(), &utf8[0],
-                        (int)utf8.size(), NULL, NULL);
-
-    return utf8;
+    try {
+        std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> conv;
+        return conv.to_bytes(str);
+    } catch (const std::exception&) {
+        return std::string();
+    }
 }
-#endif
 
 
 
@@ -797,7 +880,7 @@ Strutil::safe_strcpy(char* dst, string_view src, size_t size) noexcept
 void
 Strutil::skip_whitespace(string_view& str) noexcept
 {
-    while (str.size() && isspace(str.front()))
+    while (str.size() && Strutil::isspace(str.front()))
         str.remove_prefix(1);
 }
 
@@ -806,7 +889,7 @@ Strutil::skip_whitespace(string_view& str) noexcept
 void
 Strutil::remove_trailing_whitespace(string_view& str) noexcept
 {
-    while (str.size() && isspace(str.back()))
+    while (str.size() && Strutil::isspace(str.back()))
         str.remove_suffix(1);
 }
 
@@ -911,7 +994,7 @@ Strutil::parse_string(string_view& str, string_view& val, bool eat,
     const char *begin = p.begin(), *end = p.begin();
     bool escaped = false;  // was the prior character a backslash
     while (end != p.end()) {
-        if (isspace(*end) && !quoted)
+        if (Strutil::isspace(*end) && !quoted)
             break;  // not quoted and we hit whitespace: we're done
         if (quoted && *end == lead_char && !escaped)
             break;  // closing quote -- we're done (beware embedded quote)
