@@ -190,6 +190,14 @@ test_get_rest_arguments()
     OIIO_CHECK_EQUAL(base, "atext");
     OIIO_CHECK_EQUAL(result["arg1"], "value1");
     OIIO_CHECK_EQUAL(result["arg2"], "somevalue");
+
+    // Test windows long filename syntax
+    result.clear();
+    url = "\\\\?\\UNC\\server\\foo?arg1=value1";
+    ret = Strutil::get_rest_arguments(url, base, result);
+    OIIO_CHECK_EQUAL(ret, true);
+    OIIO_CHECK_EQUAL(base, "\\\\?\\UNC\\server\\foo");
+    OIIO_CHECK_EQUAL(result["arg1"], "value1");
 }
 
 
@@ -248,6 +256,11 @@ test_comparisons()
     OIIO_CHECK_EQUAL(Strutil::iequals("abc", "abc"), true);
     OIIO_CHECK_EQUAL(Strutil::iequals("Abc", "aBc"), true);
     OIIO_CHECK_EQUAL(Strutil::iequals("abc", "adc"), false);
+    OIIO_CHECK_EQUAL(Strutil::iequals("abc", "abcd"), false);
+    OIIO_CHECK_EQUAL(Strutil::iequals("abcd", "abc"), false);
+    OIIO_CHECK_EQUAL(Strutil::iequals("", "abc"), false);
+    OIIO_CHECK_EQUAL(Strutil::iequals("abc", ""), false);
+    OIIO_CHECK_EQUAL(Strutil::iequals("", ""), true);
 
     OIIO_CHECK_EQUAL(Strutil::starts_with("abcd", "ab"), true);
     OIIO_CHECK_EQUAL(Strutil::starts_with("aBcd", "Ab"), false);
@@ -374,8 +387,17 @@ test_comparisons()
     bench.indent (2);
     bench.units (Benchmarker::Unit::ns);
     std::string abc = "abcdefghijklmnopqrstuvwxyz";
+    std::string abcmore = "abcdefghijklmnopqrstuvwxyz1";
+    std::string abcnope = "1abcdefghijklmnopqrstuvwxyz";
     std::string haystack = std::string("begin") + abc + "oiio"
                          + Strutil::repeat(abc, 10) + "123" + abc + "end";
+    bench ("string== success", [&](){ DoNotOptimize(abc == abc); });
+    bench ("string== failure", [&](){ DoNotOptimize(abc == abcmore); });
+    bench ("iequals success", [&](){ DoNotOptimize(Strutil::iequals(abc, abc)); });
+    bench ("iless easy", [&](){ DoNotOptimize(Strutil::iless(abc, abcnope)); });
+    bench ("iless hard", [&](){ DoNotOptimize(Strutil::iless(abc, abc)); });
+    bench ("StringILess easy", [&](){ DoNotOptimize(iless(abc, abcnope)); });
+    bench ("StringILess hard", [&](){ DoNotOptimize(iless(abc, abc)); });
     bench ("contains early small", [&](){ DoNotOptimize(Strutil::contains(abc, "def")); });
     bench ("contains early big", [&](){ DoNotOptimize(Strutil::contains(haystack, "oiio")); });
     bench ("contains late small", [&](){ DoNotOptimize(Strutil::contains(abc, "uvw")); });
@@ -423,8 +445,12 @@ test_comparisons()
 
     bench ("starts_with success", [&](){ DoNotOptimize(Strutil::starts_with(abc, "abc")); });
     bench ("starts_with fail", [&](){ DoNotOptimize(Strutil::starts_with(abc, "def")); });
+    bench ("istarts_with success", [&](){ DoNotOptimize(Strutil::istarts_with(abc, "abc")); });
+    bench ("istarts_with fail", [&](){ DoNotOptimize(Strutil::istarts_with(abc, "def")); });
     bench ("ends_with success", [&](){ DoNotOptimize(Strutil::ends_with(abc, "xyz")); });
     bench ("ends_with fail", [&](){ DoNotOptimize(Strutil::ends_with(abc, "def")); });
+    bench ("iends_with success", [&](){ DoNotOptimize(Strutil::iends_with(abc, "xyz")); });
+    bench ("iends_with fail", [&](){ DoNotOptimize(Strutil::iends_with(abc, "def")); });
 }
 
 
@@ -950,9 +976,15 @@ void
 test_string_view()
 {
     std::cout << "Testing string_view methods\n";
-    std::string s("0123401234");
+    const char* cstr = "0123401234";
+    std::string s(cstr);
     string_view sr(s);
 
+    OIIO_CHECK_EQUAL(string_view(), "");  // Default ctr should be empty
+    OIIO_CHECK_EQUAL(string_view(cstr), cstr);  // Test ctr from char*
+    OIIO_CHECK_EQUAL(string_view(s), cstr);  // test ctr from std::string
+    OIIO_CHECK_EQUAL(sr, cstr);  // These better be the same
+
     OIIO_CHECK_EQUAL(sr.substr(0), sr); // whole string
     OIIO_CHECK_EQUAL(sr.substr(2), "23401234"); // nonzero pos, default n
     OIIO_CHECK_EQUAL(sr.substr(2, 3), "234"); // true substrng
